import csv
import json

import requests
from requests import HTTPError

from cruxctl.common.models.application_profile import ApplicationProfile
from cruxctl.common.data_adapters import json_to_rich_table
from cruxctl.common.models.data_format import DataFormat
from cruxctl.common.models.file_frequency import FileFrequency
from cruxctl.common.utils.api_utils import (
    MAX_PAGE_SIZE,
    get_url_based_on_profile,
    get_control_plane_url,
)

DEFAULT_API_TIMEOUT_SECONDS = 50
DEADLINE_RECOMMENDATION_EVENT_TYPE = (
    "com.crux.cp.schedule.deadline-recommendation.success.v1"
)


class DeadlineRecommendationClient:
    """
    A client for querying recommended deadlines
    """

    def get_recommended_deadline(self, profile: str, token: str, dataset_id: str):
        url = (
            f"{get_control_plane_url(profile)}"
            f"/events?filter=subject.eq.crn:dataset:{dataset_id}"
        )
        response = requests.get(
            url,
            headers={"Accept": "application/json", "Authorization": f"Bearer {token}"},
        )
        response.raise_for_status()
        returned_entries = response.json()
        entry_subject = f"crn:dataset:{dataset_id}"
        latest_entry = max(
            (
                entry
                for entry in returned_entries
                if entry.get("type") == DEADLINE_RECOMMENDATION_EVENT_TYPE
                and entry.get("subject") == entry_subject
            ),
            key=lambda x: x.get("time"),
            default=None,
        )
        deadline = latest_entry["data"] if latest_entry is not None else None
        return deadline


class DatasetDeadlineClient:
    """
    A client for handling dataset deadlines.
    """

    def get_all_deadlines(
        self,
        profile: str,
        token: str,
        dataset_id: str = None,
        limit: int = 100,
        output_format: DataFormat = None,
    ):
        """
        Retrieves all deadlines for a given dataset.

        Args:
            profile (str): The application profile.
            token (str): The authorization token.
            dataset_id (str, optional): The ID of the dataset. Defaults to None.
            output_format ([type], optional): The output format. Defaults to None.
            limit (int, optional): The limit of deadlines to retrieve. Defaults to 100.
        """
        path = "/datasets/deadlines/search"
        url = get_url_based_on_profile(profile) + path
        all_deadlines = []

        if limit < 0 or limit > MAX_PAGE_SIZE:
            limit = MAX_PAGE_SIZE

        if not dataset_id:
            payload = {}
        else:
            payload = {"datasetId": dataset_id}

        headers = {
            "accept": "*/*",
            "Content-Type": "application/json",
            "Authorization": "Bearer " + token,
        }
        page_number = 0

        while True:
            page_size = limit if 100 > limit else 100
            querystring = {"pageNumber": str(page_number), "pageSize": str(page_size)}

            response = requests.request(
                "POST",
                url,
                json=payload,
                headers=headers,
                params=querystring,
                timeout=DEFAULT_API_TIMEOUT_SECONDS,
            )

            try:
                self.raise_for_status(response)
            except HTTPError as ex:
                print(f"Error making api request: {ex}")
                break

            decoded_content = response.content.decode("utf-8")

            if decoded_content == "[]":
                break
            deadline_list = json.loads(decoded_content)
            for deadline in deadline_list:
                if "dataset" in deadline and deadline["dataset"] is not None:
                    deadline["workflowId"] = deadline["dataset"]["workflowId"]
                    deadline.pop("dataset")

            all_deadlines.extend(deadline_list)
            page_number += 1
            limit = limit - page_size
            if limit <= 0:
                break

        if output_format == DataFormat.table:
            return json_to_rich_table(all_deadlines)
        return all_deadlines

    def insert_deadline(
        self,
        profile: str,
        token: str,
        dataset_id: str,
        deadline_minute: str,
        deadline_hour: str,
        deadline_day_of_the_month: str,
        deadline_month: str,
        deadline_day_of_week: str,
        deadline_year: str,
        file_frequency: FileFrequency,
        shard: int,
        is_active: bool = True,
        is_excluded: bool = False,
        timezone: str = None,
    ):
        """
        Inserts a new deadline for a given dataset.

        Args:
            profile (str): The application profile.
            token (str): The authorization token.
            dataset_id (str): The ID of the dataset.
            deadline_minute (str): The minute of the deadline.
            deadline_hour (str): The hour of the deadline.
            deadline_day_of_the_month (str): The day of the month of the deadline.
            deadline_month (str): The month of the deadline.
            deadline_day_of_week (str): The day of the week of the deadline.
            deadline_year (str): The year of the deadline.
            file_frequency (str): The frequency of the file.
            shard (int): The shard of airflow.
            is_active (bool, optional): Whether the dataset is active. Defaults to True.
            is_excluded (bool, optional): Whether the dataset is excluded in Zendesk Exclusions.
              Defaults to False.
            timezone (str, optional): The timezone of the deadline. Defaults to None.
        """
        path = "/datasets/deadlines"
        url = get_url_based_on_profile(profile) + path

        headers = {
            "accept": "*/*",
            "Content-Type": "application/json",
            "Authorization": "Bearer " + token,
        }
        if timezone:
            payload = {
                "datasetId": dataset_id,
                "deadlineMinute": deadline_minute,
                "deadlineHour": deadline_hour,
                "deadlineDayOfTheMonth": deadline_day_of_the_month,
                "deadlineMonth": deadline_month,
                "deadlineDayOfWeek": deadline_day_of_week,
                "deadlineYear": deadline_year,
                "timezone": timezone,
                "fileFrequency": file_frequency.value,
                "shard": shard,
                "active": is_active,
                "excluded": is_excluded,
            }
        else:
            payload = {
                "datasetId": dataset_id,
                "deadlineMinute": deadline_minute,
                "deadlineHour": deadline_hour,
                "deadlineDayOfTheMonth": deadline_day_of_the_month,
                "deadlineMonth": deadline_month,
                "deadlineDayOfWeek": deadline_day_of_week,
                "deadlineYear": deadline_year,
                "fileFrequency": file_frequency.value,
                "shard": shard,
                "active": is_active,
                "excluded": is_excluded,
            }

        response = requests.request(
            "POST",
            url,
            json=payload,
            headers=headers,
            timeout=DEFAULT_API_TIMEOUT_SECONDS,
        )
        self.raise_for_status(response)

    def delete_deadline_by_id(self, profile: str, token: str, deadline_id: str):
        """
        Deletes a deadline by its ID.

        Args:
            profile (str): The application profile.
            token (str): The authorization token.
            deadline_id (str): The ID of the dataset deadline to delete.
        """
        path = f"/datasets/deadlines/{deadline_id}"
        url = get_url_based_on_profile(profile) + path

        headers = {
            "accept": "*/*",
            "Content-Type": "application/json",
            "Authorization": "Bearer " + token,
        }

        response = requests.request(
            "DELETE", url, headers=headers, timeout=DEFAULT_API_TIMEOUT_SECONDS
        )
        self.raise_for_status(response)

    def bulk_insert_deadlines(self, profile: str, token: str, file_path: str):
        """
        Bulk inserts deadlines from a CSV file.

        Args:
            profile (str): The application profile.
            token (str): The authorization token.
            file_path (str): The path to the CSV file.
        """
        path = "/datasets/deadlines/bulk-create"
        url = get_url_based_on_profile(profile) + path

        headers = {"accept": "*/*", "Authorization": "Bearer " + token}

        with open(file_path, "r") as file:
            # Read the CSV file and convert each row into a dictionary
            reader = csv.DictReader(file)
            data = list(reader)
        response = requests.request(
            "POST", url, headers=headers, json=data, timeout=DEFAULT_API_TIMEOUT_SECONDS
        )
        self.raise_for_status(response)

    def export_to_local_file(
        self, profile: ApplicationProfile, file_path: str, token: str
    ):
        """
        Exports all deadlines to a local CSV file.

        Args:
            profile (str): The application profile.
            file_path (str): The path to the CSV file.
            token(str): The authorization token.
        """
        all_deadlines = self.get_all_deadlines(
            profile, token=token, output_format=DataFormat.json, limit=-1
        )
        with open(file_path, "w") as file:
            writer = csv.DictWriter(file, fieldnames=all_deadlines[0].keys())
            writer.writeheader()
            writer.writerows(all_deadlines)

    def delete_deadlines_from_csv(self, profile: str, token: str, file_path: str):
        """
        Deletes deadlines from a CSV file by record ID.

        Args:
            profile (str): The application profile.
            token (str): The authorization token.
            file_path (str): The path to the CSV file.
        """
        with open(file_path) as csv_file:
            reader = csv.DictReader(csv_file)
            for row in reader:
                self.delete_deadline_by_id(profile, token, row["id"])

    @staticmethod
    def raise_for_status(response):
        """
        Raises `HTTPError` if one occurred.
        Checks the response for an error status and raises an exception with the error
        message provided by the server.
        :param response:
        :return:
        """

        if 400 <= response.status_code < 600:
            if isinstance(response.reason, bytes):
                try:
                    reason = response.reason.decode("utf-8")
                except UnicodeDecodeError:
                    reason = response.reason.decode("iso-8859-1")
            else:
                reason = response.reason

            raise HTTPError(
                f"{response.status_code} {reason} -> ({response.text}) for url: {response.url}",
                response=response,
            )



from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, Session


def create_pg_database_session(
    user: str, password: str, host: str, database: str, port: int = 5432
) -> Session:
    database_url = f"postgresql://{user}:{password}@{host}:{port}/{database}"

    engine = create_engine(database_url, pool_pre_ping=True)

    session = sessionmaker(autocommit=False, autoflush=False, bind=engine)

    return session()



from datetime import datetime

import requests

from cruxctl.common.utils.api_utils import (
    get_control_plane_url,
    get_api_headers,
    raise_for_status,
)

DEFAULT_API_TIMEOUT_SECONDS: int = 10
DEFAULT_CLOUD_EVENT_SOURCE: str = "cruxctl"


class DagRunHandler:
    @staticmethod
    def run_dag(
        profile: str,
        api_token: str,
        dataset_id: str,
        dag_run_id: str,
        logical_date: datetime,
        note: str,
    ) -> None:
        """
        Calls the control plane API to run a DAG for a dataset.

        :param profile: the Application Profile
        :param api_token: the API Bearer token
        :param dataset_id: ID of the dataset to run
        :param dag_run_id: user specified DAG run ID
        :param logical_date: the datetime at which to schedule the DAG run
        :param note: optional note to attach to the DAG run
        """
        base_url: str = get_control_plane_url(profile)
        url: str = f"{base_url}/datasets/{dataset_id}/run"

        headers: dict = get_api_headers(api_token)
        headers["source"] = DEFAULT_CLOUD_EVENT_SOURCE

        logical_date_str: str = logical_date.strftime("%Y-%m-%dT%H:%M:%S")

        response = requests.request(
            method="POST",
            url=url,
            headers=headers,
            timeout=DEFAULT_API_TIMEOUT_SECONDS,
            json={
                "dagRunId": dag_run_id,
                "logicalDate": logical_date_str,
                "note": note,
            },
        )

        raise_for_status(response)

    @staticmethod
    def rerun_dag(
        profile: str,
        api_token: str,
        dataset_id: str,
        dag_run_id: str,
        dry_run: bool = False,
        only_failed: bool = False,
        only_running: bool = False,
        task_ids: list[str] = None,
    ) -> None:
        """
        Calls the control plane API to rerun a DAG for a dataset.

        :param profile: the Application Profile
        :param api_token: the API Bearer token
        :param dataset_id: ID of the dataset to rerun
        :param dag_run_id: user specified DAG run ID
        :param dry_run: whether to perform a dry run
        :param only_failed: whether to only run failed tasks
        :param only_running: whether to only run running tasks
        :param task_ids: list of task IDs to run
        """
        base_url: str = get_control_plane_url(profile)
        url: str = f"{base_url}/datasets/{dataset_id}/rerun"

        headers: dict = get_api_headers(api_token)
        headers["source"] = DEFAULT_CLOUD_EVENT_SOURCE

        response = requests.request(
            method="POST",
            url=url,
            headers=headers,
            timeout=DEFAULT_API_TIMEOUT_SECONDS,
            json={
                "dagRunId": dag_run_id,
                "dryRun": dry_run,
                "onlyFailed": only_failed,
                "onlyRunning": only_running,
                "taskIds": task_ids,
            },
        )

        raise_for_status(response)



import yaml
import logging
import warnings
from pathlib import Path

import typer
from rich.console import Console
from typing_extensions import Annotated

from cruxctl.command_groups.deadlines import deadlines_snooze
from cruxctl.command_groups.deadlines.dataset_deadline_client import (
    DatasetDeadlineClient,
    DeadlineRecommendationClient,
)
from cruxctl.command_groups.profile.profile import get_current_profile
from cruxctl.common.models.data_format import DataFormat
from cruxctl.command_groups.deadlines.deadline_validations import (
    csv_file_validation,
)

from cruxctl.common.typer_constants import PROFILE_OPTION, LISTING_LIMIT_OPTION
from cruxctl.common.utils.api_utils import set_api_token
from crux_odin.dict_utils import yaml_file_to_dict
from crux_odin.types.deadlines import (
    DeadlineMinute,
    DeadlineHour,
    DeadlineDayOfMonth,
    DeadlineMonth,
    DeadlineDayOfWeek,
    DeadlineYear,
    FileFrequency,
    Timezone,
)
from cruxctl.common.utils.schedule_utils import (
    OperationMode,
    process_recommended_deadline,
    normalize_frequencies,
    partition_cron,
    delete_existing_deadline_prompt,
    create_custom_deadline,
    apply_recommended_deadline_prompt,
)

app = typer.Typer()
app.registered_commands += deadlines_snooze.app.registered_commands

console = Console()

warnings.filterwarnings("ignore")


@app.command("get-all")
def get_all(
    limit: Annotated[int, LISTING_LIMIT_OPTION] = 100,
    output_format: Annotated[
        DataFormat,
        typer.Option(
            "--output-format",
            "-o",
            case_sensitive=False,
            help="The output display format",
        ),
    ] = DataFormat.table,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Lists all delivery deadline entries
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    console.print(
        DatasetDeadlineClient().get_all_deadlines(
            profile=profile, token=token, limit=limit, output_format=output_format
        )
    )


@app.command()
def get(
    dataset_id: Annotated[str, typer.Argument(help="Dataset ID to get for")],
    output_format: Annotated[
        DataFormat,
        typer.Option(
            "--output-format",
            "-o",
            case_sensitive=False,
            help="The output display format",
        ),
    ] = DataFormat.table,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Gets the delivery deadline entries matching the dataset ID
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    console.print(
        DatasetDeadlineClient().get_all_deadlines(
            profile=profile,
            token=token,
            output_format=output_format,
            dataset_id=dataset_id,
        )
    )


@app.command()
def insert(
    file_path: Annotated[
        Path, typer.Option("--file", "-f", help="Path to the ODIN dataset spec")
    ],
    deadline_minute: Annotated[
        DeadlineMinute,
        typer.Option(
            "--deadline-minute",
            "-min",
            help="Minute of the delivery deadline. Allowed values: 0-59.",
        ),
    ] = None,
    deadline_hour: Annotated[
        DeadlineHour,
        typer.Option(
            "--deadline-hour",
            "-hr",
            help=(
                "Hour of the delivery deadline. Allowed values: 0-23. "
                "Leave * if expected every hour."
            ),
        ),
    ] = None,
    deadline_day_of_month: Annotated[
        DeadlineDayOfMonth,
        typer.Option(
            "--deadline-day-of-the-month",
            "-dom",
            help=(
                "Day of the month delivery deadline. Allowed values: 1-31. "
                "Leave * if expected every day."
            ),
        ),
    ] = None,
    deadline_month: Annotated[
        DeadlineMonth,
        typer.Option(
            "--deadline-month",
            "-m",
            help=(
                "Month of the delivery deadline. Allowed values: 1-12."
                " Leave * if expected every month."
            ),
        ),
    ] = None,
    deadline_day_of_week: Annotated[
        DeadlineDayOfWeek,
        typer.Option(
            "--deadline-day-of-week",
            "-dow",
            help=(
                "Day of the week delivery deadline. Allowed values: 0-6."
                " Leave * if expected every day."
            ),
        ),
    ] = None,
    deadline_year: Annotated[
        DeadlineYear,
        typer.Option(
            "--deadline-year",
            "-y",
            help=(
                "Selected year. Will most likely be * as"
                " deliveries are expected every year."
            ),
        ),
    ] = None,
    file_frequency: Annotated[
        FileFrequency,
        typer.Option(
            "--file-frequency",
            "-freq",
            help="Frequency of the file. Example values: daily, weekly, monthly, yearly",
        ),
    ] = None,
    timezone: Annotated[
        Timezone,
        typer.Option(
            "--timezone",
            "-tz",
            help="Timezone for the deadline",
        ),
    ] = None,
    force: Annotated[
        bool,
        typer.Option(
            "--force",
            is_flag=True,
            help=(
                "Force apply the custom deadline even if it conflicts "
                "with existing ones, and will bypass the additional "
                "recommendation sequence. Note that a custom deadline "
                "must be provided for this option to be applicable."
            ),
        ),
    ] = False,
    mode: Annotated[
        OperationMode,
        typer.Option(
            "--mode",
            help=(
                "Mode to apply the custom deadline when '--force' is used. "
                "'replace' to replace existing deadlines or 'append' to add "
                "to existing deadlines. This is only applicable if '--force' "
                "is also applied."
            ),
            case_sensitive=False,
        ),
    ] = "append",
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Inserts a single delivery deadline
    """
    custom_deadline = None
    recommended_deadline = None
    existing_deadlines = []
    recommended_deadlines = []
    if not profile:
        profile = get_current_profile()

    custom_deadline_parameters = {
        "deadline_minute": deadline_minute,
        "deadline_hour": deadline_hour,
        "deadline_day_of_month": deadline_day_of_month,
        "deadline_month": deadline_month,
        "deadline_day_of_week": deadline_day_of_week,
        "deadline_year": deadline_year,
        "file_frequency": file_frequency,
        "timezone": timezone,
    }
    if all(param is not None for param in custom_deadline_parameters.values()):
        custom_deadline = create_custom_deadline(**custom_deadline_parameters)
    elif any(param is not None for param in custom_deadline_parameters.values()):
        missing_params = [
            param
            for param, value in custom_deadline_parameters.items()
            if value is None
        ]
        if missing_params:
            raise ValueError(
                f"Missing deadline parameters: {', '.join(missing_params)}"
            )

    # Suppress logging
    logging.getLogger().setLevel(logging.ERROR)
    workflow = yaml_file_to_dict(str(file_path))
    if "availability_deadlines" in workflow:
        existing_deadlines = [
            create_custom_deadline(**deadline)
            for deadline in workflow["availability_deadlines"]
        ]
    else:
        workflow["availability_deadlines"] = existing_deadlines

    if force:
        if not custom_deadline:
            console.print(
                "Warning: --force option is set but no custom deadline is provided; "
                "the --force flag will not be applied.",
                style="yellow",
            )
        else:
            if mode == "append":
                workflow["availability_deadlines"].append(custom_deadline)
            elif mode == "replace":
                workflow["availability_deadlines"] = [custom_deadline]
    else:
        token = set_api_token(console, profile)
        deadline_client = DeadlineRecommendationClient()
        recommended_deadline = deadline_client.get_recommended_deadline(
            profile=profile,
            token=token,
            dataset_id=workflow["metadata"]["dataset_id"],
        )
        if recommended_deadline:
            file_frequency = normalize_frequencies(
                recommended_deadline.get("cadence_processing_end")
            )
            timezone = recommended_deadline.get("user_timezone")
            recommended_deadline = process_recommended_deadline(recommended_deadline)
            recommended_deadline_partitions = partition_cron(recommended_deadline)
            for recommended_deadline in recommended_deadline_partitions:
                recommended_deadline_parts = recommended_deadline.split(" ")
                recommended_deadline = create_custom_deadline(
                    deadline_minute=recommended_deadline_parts[0],
                    deadline_hour=recommended_deadline_parts[1],
                    deadline_day_of_month=recommended_deadline_parts[2],
                    deadline_month=recommended_deadline_parts[3],
                    deadline_day_of_week=recommended_deadline_parts[4],
                    deadline_year=recommended_deadline_parts[5],
                    file_frequency=file_frequency,
                    timezone=timezone,
                )
                recommended_deadlines.append(recommended_deadline)
        existing_deadlines.sort()
        recommended_deadlines.sort()

        def handle_deadlines(existing, recommended, custom_params):
            if existing:
                if delete_existing_deadline_prompt(console, existing_deadlines):
                    workflow["availability_deadlines"] = []
            if recommended:
                if apply_recommended_deadline_prompt(console, recommended_deadlines):
                    workflow["availability_deadlines"].extend(recommended_deadlines)
                    return
            console.print("Please enter the details for the new deadline:")
            custom_deadline = create_custom_deadline(**custom_params)
            workflow["availability_deadlines"].append(custom_deadline)

        handle_deadlines(
            existing_deadlines, recommended_deadlines, custom_deadline_parameters
        )
    workflow["availability_deadlines"] = [
        deadline if isinstance(deadline, dict) else deadline.__dict__
        for deadline in workflow["availability_deadlines"]
    ]
    with open(file_path, "w") as file:
        yaml.dump(workflow, file, sort_keys=False)
    console.print("[green]Deadline configuration updated successfully.[/green]")
    logging.getLogger().setLevel(logging.INFO)


@app.command("import")
def bulk_import(
    ctx: typer.Context,
    file_path: Annotated[
        Path,
        typer.Argument(
            help="""
    Path to CSV file to import.\n
    The CSV must have a header with the following columns:\n
    dataset_id, deadline_minute, deadline_hour, deadline_day_of_month,
    deadline_month, deadline_day_of_week, deadline_year, timezone.\n
    [bold yellow]WARNING[/bold yellow]: No validation is performed on the CSV file.
    It is assumed that the content is valid and entries are deduplicated.
    """
        ),
    ],
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Imports a CSV file with the delivery deadline entries.
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    csv_file_validation(ctx, profile, file_path, token=token)

    DatasetDeadlineClient().bulk_insert_deadlines(
        profile, file_path=str(file_path), token=token
    )
    console.print("[green]Import completed successfully[/green]")


@app.command("export")
def bulk_export(
    file_path: Annotated[
        Path, typer.Argument(help="Path to CSV file to write data in locally")
    ],
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Exports a CSV file with all the delivery deadlines to the provided local path
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    DatasetDeadlineClient().export_to_local_file(
        profile, file_path=str(file_path), token=token
    )
    console.print("[green]Export completed successfully[/green]")


@app.command()
def delete(
    id: Annotated[str, typer.Argument(help="ID to match for deletion")],
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Deletes the delivery deadline entries with the given ID
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    DatasetDeadlineClient().delete_deadline_by_id(profile, token, id)
    console.print(f"[green]Deleted entry with ID: {id}[/green]")


@app.command("dataset-delete")
def dataset_delete(
    dataset_id: Annotated[str, typer.Argument(help="dataset_id to match for deletion")],
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Deletes the delivery deadline entries with the given dataset_id
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    deadlines_by_dataset_id = DatasetDeadlineClient().get_all_deadlines(
        profile, token, dataset_id=dataset_id, output_format=DataFormat.json, limit=-1
    )

    console.print(deadlines_by_dataset_id)

    is_delete = typer.confirm(
        "Above are the entries matching with dataset_id, do you want to delete them?",
        default=False,
    )
    if is_delete:
        for deadline in deadlines_by_dataset_id:
            DatasetDeadlineClient().delete_deadline_by_id(
                profile, token, deadline["id"]
            )
        console.print(f"[green]Deleted entries with dataset ID: {dataset_id}[/green]")
    else:
        console.print("[red]No entries were deleted[/red]")


@app.command("bulk-delete")
def bulk_delete(
    file_path: Annotated[
        Path, typer.Argument(help="Path to CSV file to write data in locally")
    ],
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Delete all deadlines from a CSV file. The deadlines are to be defined by the
     ID of the deadline record (which is a UUID), NOT the dataset's ID. The CSV
     file should have a single "id" column with the header included.
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    DatasetDeadlineClient().delete_deadlines_from_csv(
        profile, file_path=str(file_path), token=token
    )
    console.print("[green]Deadlines deleted successfully[/green]")



import requests

from cruxctl.common.utils.api_utils import (
    get_control_plane_url,
    get_api_headers,
    raise_for_status,
)

DEFAULT_API_TIMEOUT_SECONDS: int = 10


class LogsHandler:
    @staticmethod
    def get_logs(
        profile: str,
        api_token: str,
        dataset_id: str,
        service: str,
        export_id: str,
        execution_date: str,
        delivery_id: str,
        cdu_id: str,
        page_token: str,
        page_size: int,
    ) -> dict:
        """
        Calls the control plane API to run a DAG for a dataset.

        :param profile: the Application Profile
        :param api_token: the API Bearer token
        :param dataset_id: ID of the dataset to get logs for
        :param service: optional Service name to retrieve logs for
        :param export_id: optional Export ID
        :param execution_date: optional Execution Date
        :param delivery_id: optional Delivery ID
        :param cdu_id: optional CDU ID
        :param page_token: optional page token for next page of logs
        :param page_size: optional page size for logs
        """
        base_url: str = get_control_plane_url(profile)
        url: str = f"{base_url}/datasets/{dataset_id}/logs"

        headers: dict = get_api_headers(api_token)

        response = requests.request(
            method="POST",
            url=url,
            headers=headers,
            timeout=DEFAULT_API_TIMEOUT_SECONDS,
            json={
                "datasetId": dataset_id,
                "serviceType": service.upper(),
                "exportId": export_id,
                "executionDate": execution_date,
                "deliveryId": delivery_id,
                "cduId": cdu_id,
                "pageToken": page_token,
                "pageSize": page_size,
                "orderBy": "timestamp desc",
            },
        )

        raise_for_status(response)
        return response.json()



import datetime
from collections import defaultdict

from google.cloud.bigquery.table import RowIterator
from rich.panel import Panel
from rich.text import Text
from rich.tree import Tree

from cruxctl.command_groups.dataset_health.models.dataset_health_branch import (
    DatasetHealthBranch,
)


class HealthTreeDataAdapter:
    def bigquery_to_health_tree_by_dataset(self, bq_row_iterator: RowIterator) -> Tree:
        tree = self._create_base_tree()

        for bq_row in bq_row_iterator:
            self._append_dataset_health_branch(
                tree,
                bq_row.get("dataset_id"),
                bq_row.get("aggregated_status"),
                bq_row.get("processing_status"),
                bq_row.get("delivery_deadline"),
                bq_row.get("dispatch_statuses"),
                bq_row.get("subscription_ids"),
                bq_row.get("subscribers"),
            )

        return tree

    def _create_base_tree(self) -> Tree:
        return Tree("", guide_style="bold bright_blue", hide_root=True)

    def _append_dataset_health_branch(
        self,
        tree: Tree,
        dataset_id: str,
        aggregated_status: str,
        processing_status: str,
        delivery_deadline: datetime,
        dispatch_statuses: list[str],
        subscription_ids: list[str],
        subscribers: list[str] = None,
    ):
        aggregated_health_branch = tree.add(
            Panel(
                self._get_styled_status(
                    aggregated_status,
                    f" | Dataset ID: {dataset_id} | Deadline: {delivery_deadline} ",
                ),
                title="Dataset Health",
                title_align="left",
                expand=False,
            )
        )

        processing_health_branch = aggregated_health_branch.add(
            Panel(
                self._get_styled_status(processing_status),
                title="Processing Health",
                title_align="left",
                expand=False,
            )
        )

        if subscribers:
            self._append_health_leaves(
                processing_health_branch,
                dispatch_statuses,
                subscription_ids,
                subscribers,
            )
        else:
            self._append_health_leaves_no_subscribers(
                processing_health_branch, dispatch_statuses, subscription_ids
            )

    def _append_health_leaves(
        self,
        branch: Tree,
        dispatch_statuses: list[str],
        subscription_ids: list[str],
        subscribers: list[str],
    ):
        for dispatch_status, subscription_id, subscriber in zip(
            dispatch_statuses, subscription_ids, subscribers
        ):
            styled_status = self._get_styled_status(
                dispatch_status,
                f" | Subscriber {subscriber} | Subscription ID: {subscription_id}",
            )

            leaf = Panel(
                styled_status,
                title="Dispatch Health",
                title_align="left",
                expand=False,
            )

            branch.add(leaf)

    def _append_health_leaves_no_subscribers(
        self, branch: Tree, dispatch_statuses: list[str], subscription_ids: list[str]
    ):
        for dispatch_status, subscription_id in zip(
            dispatch_statuses, subscription_ids
        ):
            styled_status = self._get_styled_status(
                dispatch_status, f" | Subscription ID: {subscription_id}"
            )

            leaf = Panel(
                styled_status,
                title="Dispatch Health",
                title_align="left",
                expand=False,
            )

            branch.add(leaf)

    def _get_styled_status(self, status: str, extra_text: str = None) -> Text:
        if status == "healthy":
            style = "bold green"
            icon = "🟢"
        elif status == "delivered_late":
            style = "bold gold1"
            icon = "🟡"
        else:
            style = "bold red"
            icon = "🔴"

        title = f"{icon} {status.title().replace('_', ' ')}"

        text = Text(title, style=style)

        if extra_text:
            text += Text(extra_text, style="grey50")

        return text

    def bigquery_to_health_tree_by_subscriber(
        self, bq_row_iterator: RowIterator
    ) -> Tree:
        tree = self._create_base_tree()

        subscriber_health_branch_multimap = defaultdict(list[DatasetHealthBranch])

        for bq_row in bq_row_iterator:
            subscriber_id = bq_row.get("subscriber_id")

            health_branch = DatasetHealthBranch(**bq_row)

            subscriber_health_branch_multimap[subscriber_id].append(health_branch)

        for subscriber_id, health_branches in subscriber_health_branch_multimap.items():
            subscriber_health_branch = tree.add(
                Panel(
                    f"{subscriber_id}",
                    title="Subscriber",
                    title_align="left",
                    expand=False,
                )
            )

            for health_branch in health_branches:
                self._append_dataset_health_branch(
                    subscriber_health_branch,
                    health_branch.dataset_id,
                    health_branch.aggregated_status,
                    health_branch.processing_status,
                    health_branch.delivery_deadline,
                    health_branch.dispatch_statuses,
                    health_branch.subscription_ids,
                )

        return tree



from dataclasses import dataclass


@dataclass
class DeadlineRow:
    dataset_id: str
    deadline_minute: str
    deadline_hour: str
    deadline_day_of_the_month: str
    deadline_month: str
    deadline_day_of_week: str
    deadline_year: str
    timezone: str
    file_frequency: str
    shard: str
    is_active: str
    is_excluded: str

    def to_key(self):
        return (
            self.dataset_id,
            self.deadline_minute,
            self.deadline_hour,
            self.deadline_day_of_the_month,
            self.deadline_month,
            self.deadline_day_of_week,
            self.deadline_year,
            self.timezone,
            self.file_frequency,
            self.shard,
            self.is_active,
            self.is_excluded,
        )



from cruxctl.common.models.application_profile import ApplicationProfile


_POSTGRES_DATABASE_LOOKUP = {
    ApplicationProfile.local: (
        "projects/crux-staging/locations/us/"
        "connections/crux-stg-pgdb-1-gusc1-read-replica"
    ),
    ApplicationProfile.staging: (
        "projects/crux-staging/locations/us/"
        "connections/crux-stg-pgdb-1-gusc1-read-replica"
    ),
    ApplicationProfile.prod: "crux-199318.us.crux-199318_us_crux-prod-pgdb-1-gusc1-replica-1",
}


_PROCESSING_HEALTH_TABLE_LOOKUP = {
    ApplicationProfile.local: "delivery_recon.test_dataset_processing_health_history",
    ApplicationProfile.staging: "delivery_recon.dataset_processing_health_history",
    ApplicationProfile.prod: "delivery_recon.dataset_processing_health_history",
}


_DISPATCH_HEALTH_TABLE_LOOKUP = {
    ApplicationProfile.local: "delivery_recon.test_dataset_dispatch_health_history",
    ApplicationProfile.staging: "delivery_recon.dataset_dispatch_health_history",
    ApplicationProfile.prod: "delivery_recon.dataset_dispatch_health_history",
}

_AGGREGATED_BY_DATASET_STATUS_VIEW_LOOKUP = {
    ApplicationProfile.local: "delivery_recon.test_aggregated_by_dataset_status_view",
    ApplicationProfile.staging: "delivery_recon.aggregated_by_dataset_status_view",
    ApplicationProfile.prod: "delivery_recon.aggregated_by_dataset_status_view",
}

_AGGREGATED_BY_SUBSCRIBER_STATUS_VIEW_LOOKUP = {
    ApplicationProfile.local: "delivery_recon.test_aggregated_dataset_by_subscriber_status_view",
    ApplicationProfile.staging: "delivery_recon.aggregated_dataset_by_subscriber_status_view",
    ApplicationProfile.prod: "delivery_recon.aggregated_dataset_by_subscriber_status_view",
}


def get_postgres_database(profile: ApplicationProfile) -> str:
    if profile not in _POSTGRES_DATABASE_LOOKUP:
        raise ValueError(f"Invalid profile: {profile}")

    return _POSTGRES_DATABASE_LOOKUP[profile]


def get_processing_health_table(profile: ApplicationProfile) -> str:
    if profile not in _PROCESSING_HEALTH_TABLE_LOOKUP:
        raise ValueError(f"Invalid profile: {profile}")

    return _PROCESSING_HEALTH_TABLE_LOOKUP[profile]


def get_dispatch_health_table(profile: ApplicationProfile) -> str:
    if profile not in _DISPATCH_HEALTH_TABLE_LOOKUP:
        raise ValueError(f"Invalid profile: {profile}")

    return _DISPATCH_HEALTH_TABLE_LOOKUP[profile]


def get_aggregated_by_dataset_status_view(profile: ApplicationProfile) -> str:
    if profile not in _AGGREGATED_BY_DATASET_STATUS_VIEW_LOOKUP:
        raise ValueError(f"Invalid profile: {profile}")

    return _AGGREGATED_BY_DATASET_STATUS_VIEW_LOOKUP[profile]


def get_aggregated_by_subscriber_status_view(profile: ApplicationProfile) -> str:
    if profile not in _AGGREGATED_BY_SUBSCRIBER_STATUS_VIEW_LOOKUP:
        raise ValueError(f"Invalid profile: {profile}")

    return _AGGREGATED_BY_SUBSCRIBER_STATUS_VIEW_LOOKUP[profile]



from sqlalchemy import text

from cruxctl.command_groups.dataset_health.models.dataset_health_profile import (
    get_processing_health_table,
    get_dispatch_health_table,
)
from cruxctl.common.models.application_profile import ApplicationProfile
from cruxctl.common.utils.database_utils import create_pg_database_session

AGGREGATED_BY_DATASET_HEALTH_VIEW = "delivery_recon.aggregated_by_dataset_status_view"
AGGREGATED_BY_SUBSCRIBER_HEALTH_VIEW = (
    "delivery_recon.aggregated_dataset_by_subscriber_status_view"
)
NOTIFICATION_HISTORY_TABLE = "delivery_recon.dataset_notification_history"


class PostgresDatasetHealthRepo:
    def __init__(self, user: str, password: str, host: str, database: str, port: int):
        self.user = user
        self.password = password
        self.host = host
        self.database = database
        self.port = port

    def delete_all_processing_health(self):
        table = get_processing_health_table(ApplicationProfile.local)
        self._truncate_table(table)

    def delete_all_dispatch_health(self):
        table = get_dispatch_health_table(ApplicationProfile.local)
        self._truncate_table(table)

    def _truncate_table(self, table: str):
        with create_pg_database_session(
            user=self.user,
            password=self.password,
            host=self.host,
            database=self.database,
            port=self.port,
        ) as session:
            session.execute(text(f"TRUNCATE TABLE {table}"))
            session.commit()



from datetime import datetime

import typer
from rich.console import Console
from typing_extensions import Annotated

from cruxctl.command_groups.deadlines.dataset_snooze_client import DatasetSnoozeClient
from cruxctl.command_groups.deadlines.deadline_validations import (
    is_snooze_duration_excessive,
)
from cruxctl.command_groups.deadlines.models.deadline_notification_snooze_units import (
    DeadlineNotificationSnoozeUnits,
)
from cruxctl.common.models.data_format import DataFormat
from cruxctl.common.typer_constants import (
    PROFILE_OPTION,
    LISTING_LIMIT_OPTION,
    SORT_BY_OPTION,
    FILTERS_OPTION,
    PAGINATION_TOKEN_OPTION,
    DATA_FORMAT_OPTION,
)
from cruxctl.common.utils.api_utils import set_api_token

app = typer.Typer()
console = Console()


@app.command("get-all-notification-snooze")
def get_all_notification_snooze(
    profile: Annotated[str, PROFILE_OPTION],
    limit: Annotated[int, LISTING_LIMIT_OPTION] = 100,
    sort_by: Annotated[str, SORT_BY_OPTION] = None,
    filters: Annotated[str, FILTERS_OPTION] = None,
    pagination_token: Annotated[str, PAGINATION_TOKEN_OPTION] = None,
    output_format: Annotated[DataFormat, DATA_FORMAT_OPTION] = DataFormat.table,
):
    """
    Lists all delivery deadline notification snooze entries
    """
    token: str = set_api_token(console, profile)

    console.print(
        DatasetSnoozeClient().get_all_notification_snooze(
            profile=profile,
            api_token=token,
            sort_by=sort_by,
            limit=limit,
            filters=filters,
            pagination_token=pagination_token,
            output_format=output_format,
        )
    )


@app.command("get-notification-snooze")
def get_notification_snooze(
    profile: Annotated[str, PROFILE_OPTION],
    dataset_id: Annotated[str, typer.Argument(help="Dataset ID to get for")],
    output_format: Annotated[DataFormat, DATA_FORMAT_OPTION] = DataFormat.table,
):
    """
    Gets the delivery deadline notification snooze entries matching the dataset ID
    """
    token: str = set_api_token(console, profile)

    console.print(
        DatasetSnoozeClient().get_notification_snooze_by_dataset_id(
            profile=profile,
            api_token=token,
            output_format=output_format,
            dataset_id=dataset_id,
        )
    )


@app.command("create-notification-snooze")
def create_notification_snooze(
    profile: Annotated[str, PROFILE_OPTION],
    dataset_id: Annotated[
        str, typer.Argument(help="Dataset ID to snooze notifications for")
    ],
    snooze_duration: Annotated[
        int, typer.Argument(help="Defines how long to snooze for", min=1)
    ],
    snooze_units: Annotated[
        DeadlineNotificationSnoozeUnits,
        typer.Argument(help="Units for the snooze duration.", case_sensitive=False),
    ],
    snooze_start_time: Annotated[
        datetime,
        typer.Argument(
            formats=["%Y-%m-%dT%H:%M:%S"],
            help="The datetime (in UTC) at which the snooze will start on.\n"
            "If not defined, will start immediately.",
        ),
    ] = None,
):
    """
    Creates a notification snooze for the delivery deadline entries with the given dataset ID.
    """
    if is_snooze_duration_excessive(snooze_duration, snooze_units):
        is_proceed = typer.confirm(
            "The snooze duration is more than 2 weeks. Do you want to proceed?",
            default=False,
        )
        if not is_proceed:
            return

    token: str = set_api_token(console, profile)

    existing_snooze_table = DatasetSnoozeClient().get_notification_snooze_by_dataset_id(
        profile=profile,
        api_token=token,
        output_format=DataFormat.table,
        dataset_id=dataset_id,
    )

    if existing_snooze_table.row_count > 0:
        console.print(existing_snooze_table)

        is_proceed = typer.confirm(
            f"There are existing notification snoozes with dataset ID: {dataset_id}. "
            f"Creating a new one will delete the existing ones. Do you want to proceed?",
            default=False,
        )
        if not is_proceed:
            return

        DatasetSnoozeClient().delete_notification_snooze_by_dataset_id(
            profile=profile, api_token=token, dataset_id=dataset_id
        )
        console.print(
            f"[green]Deleted notification snooze with dataset ID: {dataset_id}[/green]"
        )

    DatasetSnoozeClient().insert_notification_snooze(
        profile=profile,
        api_token=token,
        dataset_id=dataset_id,
        snooze_duration=snooze_duration,
        snooze_units=snooze_units,
        snooze_start_time=snooze_start_time,
    )

    console.print("[green]Notification snooze created successfully[/green]")


@app.command("delete-notification-snooze")
def delete_notification_snooze(
    profile: Annotated[str, PROFILE_OPTION],
    dataset_id: Annotated[
        str, typer.Argument(help="Dataset ID to match for deleting notification snooze")
    ],
):
    """
    Deletes the delivery deadline notification snooze entry with the given dataset ID
    """
    token: str = set_api_token(console, profile)
    DatasetSnoozeClient().delete_notification_snooze_by_dataset_id(
        profile=profile, api_token=token, dataset_id=dataset_id
    )
    console.print(
        f"[green]Deleted notification snooze with dataset ID: {dataset_id}[/green]"
    )


@app.command("delete-expired-notification-snooze")
def delete_expired_notification_snooze(
    profile: Annotated[str, PROFILE_OPTION],
):
    """
    Deletes all expired delivery deadline notification snooze entries
    """
    token: str = set_api_token(console, profile)
    DatasetSnoozeClient().delete_expired_notification_snoozes(
        profile=profile, api_token=token
    )
    console.print(
        "[green]Deleted all expired notification snoozes successfully[/green]"
    )


@app.command("update-notification-snooze")
def update_notification_snooze(
    profile: Annotated[str, PROFILE_OPTION],
    dataset_id: Annotated[
        str, typer.Argument(help="Dataset ID to match for deleting notification snooze")
    ],
    snooze_id: Annotated[
        str, typer.Argument(help="Snooze ID to match for deleting notification snooze")
    ],
    snooze_duration: Annotated[
        int, typer.Argument(help="Defines how long to snooze for", min=1)
    ],
    snooze_units: Annotated[
        DeadlineNotificationSnoozeUnits,
        typer.Argument(help="Units for the snooze duration.", case_sensitive=False),
    ],
    snooze_start_time: Annotated[
        datetime,
        typer.Argument(
            formats=["%Y-%m-%dT%H:%M:%S"],
            help="The datetime (in UTC) at which the snooze will start on. "
            "If not defined, will start immediately.",
        ),
    ] = None,
):
    """
    Deletes all expired delivery deadline notification snooze entries
    """
    token: str = set_api_token(console, profile)
    DatasetSnoozeClient().update_notification_snooze(
        profile=profile,
        dataset_id=dataset_id,
        api_token=token,
        snooze_id=snooze_id,
        snooze_duration=snooze_duration,
        snooze_units=snooze_units,
        snooze_start_time=snooze_start_time,
    )
    console.print("[green]Updated notification snooze successfully[/green]")


if __name__ == "__main__":
    app()



import typer
import itertools
from enum import Enum
from typing import Dict, Any, List
from rich.console import Console
from crux_odin.dataclass import AvailabilityDeadline
from crux_odin.types.deadlines import (
    DeadlineMinute,
    DeadlineHour,
    DeadlineDayOfMonth,
    DeadlineMonth,
    DeadlineDayOfWeek,
    DeadlineYear,
    FileFrequency,
    Timezone,
)


class OperationMode(str, Enum):
    """
    The operation mode for command execution.
    """

    append = "append"
    replace = "replace"


def process_recommended_deadline(recommended_deadline: Dict[str, Any]) -> str:
    last_available_end = recommended_deadline.get("latest_time_processing_end")
    cron_end = recommended_deadline.get("schedule_processing_end")
    if last_available_end is not None:
        deadline_cron_end = set_cron_time(cron_end, last_available_end) + " *"
    else:
        deadline_cron_end = cron_end + " *"
    return deadline_cron_end


def normalize_frequencies(cadence: str) -> str:
    cadence_to_file_frequency = {
        "annually": "yearly",
        "quarterly": "semi-annual",
        "monthly": "monthly",
        "weekly": "weekly",
        "daily": "daily",
        "hourly": "intraday",
        "minute_30": "intraday",
        "minute_15": "intraday",
    }
    return cadence_to_file_frequency.get(cadence)


def set_cron_time(cron: str, time_string: str) -> str:
    cron_parts = cron.split(" ")

    time_parts = time_string.split(":")

    cron_parts[0] = str(int(time_parts[1]))
    cron_parts[1] = str(int(time_parts[0]))

    new_cron = " ".join(cron_parts)

    return new_cron


def partition_cron(cron: str) -> list:
    """
    Splits a cron string into all possible combinations for each field.

    For example, a cron string like "1,2 * * * *" will be split into
    ["1 * * * *", "2 * * * *"].

    This function will perform this operation for each field in the cron string.

    Args:
        cron (str): The cron string to split.

    Returns:
        List[str]: A list of cron strings with all combinations of fields.
    """
    cron_parts = cron.split(" ")
    cron_fields = [part.split(",") if "," in part else [part] for part in cron_parts]
    cron_combinations = list(itertools.product(*cron_fields))
    return [" ".join(combination) for combination in cron_combinations]


def delete_existing_deadline_prompt(
    console: Console, existing_deadlines: List[AvailabilityDeadline]
) -> bool:
    console.print("Existing Deadline(s):")
    console.print(existing_deadlines)
    is_delete = typer.confirm(
        "There are existing entries for this datasetID. Do you want to delete them "
        "before inserting?",
        default=True,
    )
    return is_delete


def create_custom_deadline(
    deadline_minute: DeadlineMinute = None,
    deadline_hour: DeadlineHour = None,
    deadline_day_of_month: DeadlineDayOfMonth = None,
    deadline_month: DeadlineMonth = None,
    deadline_day_of_week: DeadlineDayOfWeek = None,
    deadline_year: DeadlineYear = None,
    file_frequency: FileFrequency = None,
    timezone: Timezone = None,
) -> AvailabilityDeadline:
    if deadline_minute is None:
        deadline_minute = DeadlineMinute(typer.prompt("Enter deadline minute"))

    if deadline_hour is None:
        deadline_hour = DeadlineHour(typer.prompt("Enter deadline hour"))

    if deadline_day_of_month is None:
        deadline_day_of_month = DeadlineDayOfMonth(
            typer.prompt("Enter deadline day of the month")
        )

    if deadline_month is None:
        deadline_month = DeadlineMonth(typer.prompt("Enter deadline month"))

    if deadline_day_of_week is None:
        deadline_day_of_week = DeadlineDayOfWeek(
            typer.prompt("Enter deadline day of the week")
        )

    if deadline_year is None:
        deadline_year = DeadlineYear(typer.prompt("Enter deadline year"))

    if file_frequency is None:
        file_frequency = FileFrequency(typer.prompt("Enter file frequency"))

    if timezone is None:
        timezone = Timezone(typer.prompt("Enter timezone"))

    custom_deadline = AvailabilityDeadline(
        deadline_minute=deadline_minute,
        deadline_hour=deadline_hour,
        deadline_day_of_month=deadline_day_of_month,
        deadline_month=deadline_month,
        deadline_day_of_week=deadline_day_of_week,
        deadline_year=deadline_year,
        file_frequency=file_frequency,
        timezone=timezone,
    )

    return custom_deadline


def apply_recommended_deadline_prompt(
    console: Console, recommended_deadlines: List[AvailabilityDeadline]
) -> bool:
    console.print("Recommended Deadline(s):")
    console.print(recommended_deadlines)
    is_apply = typer.confirm(
        "There are recommended deadlines for this datasetID. Do you want to apply them?",
        default=True,
    )
    return is_apply



import typer
from enum import Enum


# Step 1: Define an Enum for profiles
class ProfileEnum(str, Enum):
    DEV = "dev"
    STAGING = "stg"
    PROD = "prod"


# Step 3: Function to validate a profile
def validate_profile(profile: str) -> ProfileEnum:
    try:
        return ProfileEnum(profile)
    except ValueError:
        raise typer.BadParameter("Profile must be one of 'dev', 'stg', or 'prod'.")



from dataclasses import dataclass
from datetime import datetime


@dataclass
class DatasetHealthBranch:
    subscriber_id: str
    dataset_id: str
    aggregated_status: str
    delivery_deadline: datetime
    processing_status: str
    dispatch_statuses: list[str]
    subscription_ids: list[str]



import base64
import json
import os
from typing import Dict, Any
import yaml

import requests
import platformdirs
from requests import HTTPError, Response
from rich.console import Console

from cruxctl.common.constants import default_profile

MAX_PAGE_SIZE: int = 1000


def read_config_file() -> Dict[str, Any]:
    # Check for platform dirs crux config.yaml
    config_file_path = get_config_file_path()
    if os.path.exists(config_file_path):
        with open(config_file_path, "r") as file:
            existing_file = yaml.safe_load(file)
            return existing_file if existing_file else {}
    else:
        return {}


def get_config_file_path() -> str:
    crux_config_dir = platformdirs.user_config_dir("crux")
    return os.path.join(crux_config_dir, "config.yaml")


def get_api_base_url(profile: str) -> str:
    """
    Get the base URL for the control plane REST calls.
    """
    env_url = os.environ.get("CONTROL_PLANE_URL")
    if env_url:
        return env_url

    config = read_config_file()

    if "CONTROL_PLANE_URL" in config.get(profile, {}):
        return config[profile]["CONTROL_PLANE_URL"]

    return "https://api.cruxinformatics.com"


def get_control_plane_url(profile: str) -> str:
    """
    Get the URL for the control plane REST calls.
    """
    return f"{get_api_base_url(profile)}/ops/control-plane"


def get_url_based_on_profile(
    profile: str, version: str = "v2", swap_ops_version: bool = False
) -> str:
    ops = "ops"
    if swap_ops_version:
        # The Crux API CHANGED THE ORDER OF OPS AND THE VERSION in the new version of the API.
        # For example, we have the old call /v4/ops/data-products and the new call
        # /ops/v4/data-products/maps. I talked to Naveen and that's the way it is.
        ops = version
        version = "ops"
    return f"{get_api_base_url(profile)}/{version}/{ops}"


def get_document_url(
    profile: str, version: str = "v2", swap_ops_version: bool = False
) -> str:
    """
    Get the URL for document upload
    """
    return f"{get_url_based_on_profile(profile, version, swap_ops_version)}/documents?active=true&"


def get_data_product_url(
    profile: str, version: str = "v2", swap_ops_version: bool = False
) -> str:
    """
    Get the URL for document upload
    """
    return (
        f"{get_url_based_on_profile(profile, version, swap_ops_version)}/data-products"
    )


def get_api_headers(token: str) -> dict[str, str]:
    headers = {
        "accept": "*/*",
        "Content-Type": "application/json",
        "Authorization": "Bearer " + token,
    }
    return headers


def set_api_token(console: Console, profile) -> str:
    env_token = os.environ.get("CRUX_API_TOKEN", None)

    if not env_token:
        config = read_config_file()

        if config != {}:
            profile = config.get(profile, {})
            env_token = profile.get("CRUX_API_TOKEN", None)
            if not env_token:
                raise Exception(
                    "CRUX_API_TOKEN not found for profile. Please run 'cruxctl auth login' first."
                )
            else:
                return env_token
        else:
            raise Exception(
                "No config found for profile. Please run 'cruxctl auth login' first."
            )


def raise_for_status(response: Response):
    """
    Checks the HTTP response for an error status and raises an HTTPError if one occurred.

    Args:
        response (Response): The HTTP response to check.

    Raises:
        HTTPError: If the response contains an error status (400-599).
    """
    if 400 <= response.status_code < 600:
        if isinstance(response.reason, bytes):
            try:
                reason = response.reason.decode("utf-8")
            except UnicodeDecodeError:
                reason = response.reason.decode("iso-8859-1")
        else:
            reason = response.reason

        raise HTTPError(
            f"{response.status_code} {reason} -> ({response.text}) for url: {response.url}",
            response=response,
        )


def get_auth_payload(auth: str) -> Dict[str, Any]:
    """
    Get auth token payload
    :param auth: auth token
    """
    auth and isinstance(auth, str)

    token = auth.replace("Bearer ", "")
    auth_token_components = token.split(".")
    # If you look at an auth token, it is split in 3 parts via a dot.
    # The first part is the header, the second part is the payload,
    # and the third part is the signature. We ignore the header and signature.
    if len(auth_token_components) != 3:
        raise Exception("Invalid token")
    payload_str = base64.urlsafe_b64decode(auth_token_components[1] + "==").decode()
    # Convert from string to dict.
    return json.loads(payload_str)


def get_user_info_by_email(
    email: str, access_token: str, profile: str = default_profile
) -> Dict[str, Any]:
    """
    Get user info by email from identity service
    :param email: email of the user to look up.
    :param access_token: OAuth access token.
    """
    email and isinstance(email, str)
    access_token and isinstance(access_token, str)

    rest_url = f"{get_url_based_on_profile(profile)}/user/profile/{email}"
    headers = {
        "authorization": f"Bearer {access_token}",
        "Content-Type": "application/json",
    }
    response = requests.get(rest_url, headers=headers)
    response.raise_for_status()
    return response.json()


def get_user_info_by_token(
    access_token: str, profile: str = default_profile
) -> Dict[str, Any]:
    """
    Get user info by email from identity service
    :param email: email of the user to look up.
    :param access_token: OAuth access token.
    """
    access_token and isinstance(access_token, str)

    rest_url = f"{get_url_based_on_profile(profile)}/user/profile"
    headers = {
        "authorization": f"Bearer {access_token}",
        "Content-Type": "application/json",
    }
    response = requests.get(rest_url, headers=headers)
    response.raise_for_status()
    return response.json()



import os
from cruxctl.common.utils.gcp_secrets_manager import get_secret_value


def set_openai_token(console):
    token = os.environ.get("OPENAI_API_KEY", None)
    if not token:
        secret_proj = "crux-data-science"
        secret_key = "OPENAI_API_KEY"
        console.rule("[bold red]Remove before public release", style="red")
        console.print(f"Fetching GCP secret: {secret_proj}/{secret_key}")
        console.rule(style="red")
        token = get_secret_value(secret_proj, secret_key)
        os.environ["OPENAI_API_KEY"] = token
    return token



import json
from datetime import datetime

import requests
from requests import HTTPError
from rich.table import Table

from cruxctl.command_groups.deadlines.models.deadline_notification_snooze_units import (
    DeadlineNotificationSnoozeUnits,
)
from cruxctl.common.data_adapters import json_to_rich_table
from cruxctl.common.models.data_format import DataFormat
from cruxctl.common.utils.api_utils import (
    get_api_headers,
    get_url_based_on_profile,
    raise_for_status,
    MAX_PAGE_SIZE,
)

DEFAULT_API_TIMEOUT_SECONDS: int = 50


class DatasetSnoozeClient:
    """
    A client for handling dataset snoozes.
    """

    @staticmethod
    def get_all_notification_snooze(
        profile: str,
        api_token: str,
        sort_by: str = None,
        limit: int = 50,
        filters: str = None,
        pagination_token: str = None,
        output_format: DataFormat = None,
    ) -> Table | dict:
        """
        Retrieves all snoozes from the database, paginated.
        :param pagination_token: (str): The token for pagination.
        :param profile: (str): The application profile.
        :param api_token: (str): The authorization token
        :param sort_by: (str): String specifying sort preference.
        :param limit: (int): The maximum of snoozes to retrieve. Defaults to 50.
        :param filters: (str): String specifying filter preference.
        :param output_format: (DataFormat): The output format of the data.
        :return:
        """
        path: str = "/datasets/snoozes"
        url: str = get_url_based_on_profile(profile) + path

        if limit < 0 or limit > MAX_PAGE_SIZE:
            limit = MAX_PAGE_SIZE

        headers: dict = get_api_headers(api_token)

        while True:
            querystring = {
                "pageSize": str(limit),
                "filters": filters,
                "sortBy": sort_by,
                "pageToken": pagination_token,
            }

            response = requests.request(
                method="GET",
                url=url,
                headers=headers,
                params=querystring,
                timeout=DEFAULT_API_TIMEOUT_SECONDS,
            )

            try:
                raise_for_status(response)
            except HTTPError as ex:
                print(f"Error making api request: {ex}")
                break

            decoded_content = response.content.decode("utf-8")

            if decoded_content == "[]":
                break

            paginated_snooze_list: dict = json.loads(decoded_content)

            snoozes: list = paginated_snooze_list.get("items", [])
            next_page_token = paginated_snooze_list.get("nextPageToken", None)

            if next_page_token is None:
                break
            else:
                pagination_token = next_page_token

        if output_format == DataFormat.table:
            return json_to_rich_table(snoozes)
        else:
            return paginated_snooze_list

    @staticmethod
    def get_notification_snooze_by_dataset_id(
        profile: str,
        output_format: DataFormat,
        api_token: str,
        dataset_id: str,
    ) -> Table | list:
        path: str = f"/datasets/{dataset_id}/snoozes"
        url: str = get_url_based_on_profile(profile) + path
        headers: dict = get_api_headers(api_token)

        response = requests.request(
            method="GET",
            url=url,
            headers=headers,
            timeout=DEFAULT_API_TIMEOUT_SECONDS,
        )

        try:
            raise_for_status(response)
        except HTTPError as ex:
            if ex.response.status_code == 404:
                return json_to_rich_table([])
            print(f"Error making api request: {ex}")

        decoded_content = response.content.decode("utf-8")
        snoozes: list = json.loads(decoded_content)

        if len(snoozes) != 0 and output_format == DataFormat.table:
            return json_to_rich_table(snoozes)
        else:
            return snoozes

    @staticmethod
    def insert_notification_snooze(
        profile: str,
        api_token: str,
        dataset_id: str,
        snooze_duration: int,
        snooze_units: DeadlineNotificationSnoozeUnits,
        snooze_start_time: datetime = None,
    ) -> None:
        path: str = f"/datasets/{dataset_id}/snoozes"
        url: str = get_url_based_on_profile(profile) + path
        headers: dict = get_api_headers(api_token)

        response = requests.request(
            method="POST",
            url=url,
            headers=headers,
            timeout=DEFAULT_API_TIMEOUT_SECONDS,
            json={
                "snoozeDuration": snooze_duration,
                "snoozeUnits": snooze_units,
                "snoozeStartTime": snooze_start_time,
            },
        )

        try:
            raise_for_status(response)
        except HTTPError as ex:
            print(f"Error making api request: {ex}")

    @staticmethod
    def delete_notification_snooze_by_dataset_id(
        profile: str, api_token: str, dataset_id: str
    ) -> None:
        path: str = f"/datasets/{dataset_id}/snoozes"
        url: str = get_url_based_on_profile(profile) + path
        headers: dict = get_api_headers(api_token)

        response = requests.request(
            method="DELETE",
            url=url,
            headers=headers,
            timeout=DEFAULT_API_TIMEOUT_SECONDS,
        )

        try:
            raise_for_status(response)
        except HTTPError as ex:
            print(f"Error making api request: {ex}")

    @staticmethod
    def delete_expired_notification_snoozes(profile: str, api_token: str) -> None:
        path: str = "/datasets/snoozes/expired"
        url: str = get_url_based_on_profile(profile) + path
        headers: dict = get_api_headers(api_token)

        response = requests.request(
            method="DELETE",
            url=url,
            headers=headers,
            timeout=DEFAULT_API_TIMEOUT_SECONDS,
        )

        try:
            raise_for_status(response)
        except HTTPError as ex:
            print(f"Error making api request: {ex}")

    @staticmethod
    def update_notification_snooze(
        profile: str,
        api_token: str,
        dataset_id: str,
        snooze_id: str,
        snooze_duration: int,
        snooze_units: DeadlineNotificationSnoozeUnits,
        snooze_start_time: datetime = None,
    ) -> None:
        path: str = f"/datasets/{dataset_id}/snoozes/{snooze_id}"
        url: str = get_url_based_on_profile(profile) + path
        headers: dict = get_api_headers(api_token)

        response = requests.request(
            method="PUT",
            url=url,
            headers=headers,
            timeout=DEFAULT_API_TIMEOUT_SECONDS,
            json={
                "snoozeDuration": snooze_duration,
                "snoozeUnits": snooze_units,
                "snoozeStartTime": snooze_start_time,
            },
        )

        try:
            raise_for_status(response)
        except HTTPError as ex:
            print(f"Error making api request: {ex}")



from enum import Enum


class FileFrequency(Enum):
    """
    The frequency of the file delivery.
    """

    intraday = "intraday"
    daily = "daily"
    weekly = "weekly"
    bi_weekly = "bi-weekly"
    monthly = "monthly"
    quarterly = "quarterly"
    semi_annual = "semi-annual"
    yearly = "yearly"



import warnings
from pathlib import Path

import typer
from rich.console import Console

from cruxctl.command_groups.auth.auth_utilities import validate_profile
from typing_extensions import Annotated
from cruxctl.common.typer_constants import PROFILE_OPTION
import os
import yaml
from cruxctl.common.utils.api_utils import (
    get_user_info_by_token,
    set_api_token,
    read_config_file,
    get_config_file_path,
)
from cruxctl.command_groups.auth.auth_utilities import ProfileEnum
from cruxctl.command_groups.profile.profile import (
    get_current_profile,
    unset_active_profile_in_env,
    load_active_profile_env_file,
    PROFILE_ENV_VAR,
)

app = typer.Typer()

console = Console()

warnings.filterwarnings("ignore")

env_url_map = {
    ProfileEnum.DEV: "https://app.dev.cruxdata.com/settings/api-keys",
    ProfileEnum.STAGING: "https://app.stg.cruxdata.com/settings/api-keys",
    ProfileEnum.PROD: "https://app.cruxdata.com/settings/api-keys",
}
control_plane_url_map = {
    ProfileEnum.DEV: "https://api.dev.cruxinformatics.com",
    ProfileEnum.STAGING: "https://api.stg.cruxinformatics.com",
    ProfileEnum.PROD: "https://api.cruxinformatics.com",
}


@app.command("whoami")
def check_auth(
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Check if the CLI is authenticated.
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)
    user_profile = get_user_info_by_token(token, profile)
    if user_profile:
        console.print("[green]Authenticated[/green]")
        console.print(user_profile)
    else:
        console.print("[red]Not authenticated[/red]")
        console.print("Please run 'cruxctl auth login' first.")


@app.command("login")
def authenticate_cli(
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Configure the CLI with your Crux API token.
    Choose one of 3 profiles - dev, stg, prod.
    """
    # If not found, direct the user to https://app.dev.cruxdata.com/settings/api-keys

    # Interactively accept the control plane URL from
    # the user or default to https://api.cruxinformatics.com
    if not profile:
        load_active_profile_env_file()
        current_profile = os.getenv(PROFILE_ENV_VAR)
        if not current_profile:
            console.print("[red]No profile set. Defaulting to prod. ")
            profile = ProfileEnum.PROD.value
        else:
            profile = current_profile

    validate_profile(profile)
    console.print(f"Configuring CLI for {profile} profile")
    control_plane_url = control_plane_url_map.get(profile)
    if not control_plane_url:
        console.print(
            "[red]Control plane URL not found."
            "Please make sure profile is either dev, stg or prod[/red]"
        )
        raise typer.Exit(code=1)

    console.print(
        f"Please visit {env_url_map.get(profile, 'https://app.cruxdata.com/settings/api-keys')}",
        " to get your API token",
    )

    # Interactively accept the token from the user and save to the config.yaml
    api_token = console.input("Enter your CRUX API token: ")
    config = read_config_file()

    # If the token is already present, prompt the user to overwrite the token
    if config and profile in config:
        if api_token == config[profile]["CRUX_API_TOKEN"]:
            overwrite = console.input("Token already exists. Overwrite? (y/n): ")
            if overwrite.lower() != "y":
                return

    config[profile] = {}
    config[profile]["CRUX_API_TOKEN"] = api_token
    config[profile]["CONTROL_PLANE_URL"] = control_plane_url
    config_file_path = get_config_file_path()
    os.makedirs(Path(config_file_path).parent, exist_ok=True)
    with open(config_file_path, "w") as file:
        yaml.safe_dump(config, file)

    console.print("[green]API token saved successfully[/green]")


@app.command("logout")
def logout(
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Remove the API token from the config file and environment variables.
    """
    unset_active_profile: bool = False
    if not profile:
        profile = get_current_profile()
        unset_active_profile = True
    elif profile == get_current_profile():
        unset_active_profile = True

    config = read_config_file()
    if config and profile in config:
        del config[profile]
        config_file_path = get_config_file_path()
        with open(config_file_path, "w") as file:
            yaml.safe_dump(config, file)

        if unset_active_profile:
            unset_active_profile_in_env()
        console.print(f"[green]Logged out successfully for profile {profile}[/green]")
    else:
        console.print(
            f"[red]Not logged in. No API token found for this profile:{profile}[/red]"
        )
